/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <media_content.h>
#include <app_debug.h>
#include <app_media.h>
#include <gridmgr.h>
#include <layoutmgr.h>
#include <viewmgr.h>

#include "define.h"
#include "view.h"
#include "data/mediadata.h"
#include "util/listmgr.h"
#include "util/progressbar.h"
#include "util/util.h"

#define LIST_MEDIA_COND "media_type=1 AND copyright NOT LIKE \"Unknown\""

#define TEXT_NOCONTENT "No Movies"
#define TEXT_RECENTLY_WATCHED "Recently watched"

#define GRID_PADDING 26
#define GRID_ITEM_X (378 + GRID_PADDING)
#define GRID_ITEM_Y (294 + GRID_PADDING)
#define GRID_NUM_ITEM 2

#define BOX_PADDING (62 - GRID_PADDING)

#define IMAGE_FAVORITE_ALPHA 150

struct _priv {
	Evas_Object *base;
	Evas_Object *layout;
	Evas_Object *box;
	Evas_Object *menu_btn;

	layoutmgr *lmgr;

	struct listmgr *listmgr;
	struct listmgr_data *ldata;

	struct mediadata *md;

	struct progressbar *prog;

	Eina_List *media_list;
	int cur_index;

	app_media *recent_info;
};

static char *_grid_text_get(void *data, Evas_Object *obj, const char *part)
{
	app_media *am;
	app_media_info *info;

	if (!data)
		return NULL;

	am = data;

	if (!strcmp(part, PART_ELM_TEXT_TITLE)) {
		info = app_media_get_info(am);
		if (!info) {
			_ERR("failed to get media info");
			return NULL;
		}

		return strdup(info->title);
	}

	return NULL;
}

static Evas_Object *_grid_content_get(void *data,
			Evas_Object *obj, const char *part)
{
	Evas_Object *image;
	app_media *am;
	app_media_info *info;

	if (!data)
		return NULL;

	am = data;
	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get media info");
		return NULL;
	}

	image = NULL;
	if (!strcmp(part, PART_ELM_SWALLOW_THUMBNAIL)) {
		image = util_add_image(obj, info->thumbnail_path);
		if (!image) {
			_ERR("failed to create image object");
			return NULL;
		}

		evas_object_show(image);
	} else if (!strcmp(part, PART_ELM_SWALLOW_FAVORITE)) {
		if (!info->favorite)
			return NULL;

		image = util_add_image(obj, IMAGE_THUMBNAIL_FAVORITE);
		if (!image) {
			_ERR("failed to create image object");
			return NULL;
		}

		util_set_alpha_color(image, IMAGE_FAVORITE_ALPHA);

		evas_object_show(image);
	}

	return image;
}

static struct grid_class _gclass = {
	.item_style = STYLE_GRID_MOVIE_ITEM,
	.text_get = _grid_text_get,
	.content_get = _grid_content_get
};

static bool _update_recent(void *data, Evas_Object *base)
{
	Evas_Object *recent, *image;
	app_media_info *info;
	struct tm tm;
	struct _priv *priv;
	char buf[32];

	if (!data || !base) {
		_ERR("invalid argument");
		return false;
	}

	priv = data;

	recent = elm_object_part_content_get(base, PART_ITEM_CONTENT);
	if (!recent) {
		_ERR("failed to get recent content part");
		return false;
	}

	image = elm_object_part_content_get(recent,
				PART_RECENT_CONTENT_THUMBNAIL);
	if (!image) {
		_ERR("failed to get image object");
		return false;
	}

	info = app_media_get_info(priv->recent_info);
	if (!info) {
		_ERR("failed to get app media info");
		return false;
	}

	elm_image_file_set(image, info->thumbnail_path, NULL);
	elm_image_aspect_fixed_set(image, EINA_FALSE);

	elm_object_part_text_set(recent, PART_RECENT_CONTENT_TITLE,
				info->title);

	localtime_r(&info->played_time, &tm);
	strftime(buf, sizeof(buf), "%Y.%m.%d", &tm);
	elm_object_part_text_set(recent, PART_RECENT_CONTENT_DATE, buf);

	if (info->favorite)
		elm_object_signal_emit(recent, SIG_RECENT_SHOW_FAV,
					SIG_SOURCE_SRC);
	else
		elm_object_signal_emit(recent, SIG_RECENT_HIDE_FAV,
					SIG_SOURCE_SRC);

	progressbar_reset(priv->prog, info->video->position,
				info->video->duration);

	progressbar_show(priv->prog);

	return true;
}

static bool _draw_recent_title(Evas_Object *base)
{
	Evas_Object *btn;

	btn = elm_button_add(base);
	if (!btn) {
		_ERR("failed to create button object");
		return false;
	}

	elm_object_style_set(btn, STYLE_BTN_INDEX);
	elm_object_text_set(btn, TEXT_RECENTLY_WATCHED);

	elm_object_part_content_set(base, PART_ITEM_TITLE, btn);

	return true;
}

static bool _draw_recent_content(struct _priv *priv, Evas_Object *base)
{
	Evas_Object *btn, *image;
	struct progressbar *prog;

	btn = elm_button_add(base);
	if (!btn) {
		_ERR("failed to create button object");
		return false;
	}

	elm_object_style_set(btn, STYLE_BTN_RECENT_CONTENT);

	image = elm_image_add(btn);
	if (!image) {
		_ERR("failed to create image object");
		return false;
	}

	elm_object_part_content_set(btn, PART_RECENT_CONTENT_THUMBNAIL, image);

	elm_object_part_content_set(base, PART_ITEM_CONTENT, btn);

	prog = progressbar_create(btn, STYLE_BASE_PROGRESS);
	if (!prog) {
		_ERR("failed to create progressbar");
		return false;
	}

	progressbar_set_parts(prog, PART_RECENT_CONTENT_SLIDER,
				PART_RECENT_CONTENT_TOTAL,
				PART_RECENT_CONTENT_PROGRESS, "");

	priv->prog = prog;

	return true;
}

static bool _draw_recent(void *data, Evas_Object *base)
{
	struct _priv *priv;

	if (!data || !base) {
		_ERR("invalid argument");
		return false;
	}

	priv = data;

	if (!_draw_recent_title(base)) {
		_ERR("failed to draw recent title");
		return false;
	}

	if (!_draw_recent_content(priv, base)) {
		_ERR("failed to draw recent content");
		return false;
	}

	return true;
}

static void _item_selected_cb(struct _priv *priv, app_media *am)
{
	struct view_update_data vdata;

	vdata.list = mediadata_get_medialist(priv->md);
	vdata.index = util_get_media_index(vdata.list, am);
	priv->cur_index = vdata.index;

	viewmgr_update_view(VIEW_VIEWER, UPDATE_CONTENT, &vdata);
	viewmgr_push_view(VIEW_VIEWER);
}

static void _recent_selected_cb(void *data, Evas_Object *obj)
{
	struct _priv *priv;

	if (!data || !obj)
		return;

	priv = data;

	_item_selected_cb(priv, priv->recent_info);
}

static void _grid_selected_cb(void *data, Elm_Object_Item *it)
{
	app_media *am;
	struct _priv *priv;

	if (!data || !it) {
		_ERR("invalid argument");
		return;
	}

	priv = data;

	am = elm_object_item_data_get(it);
	if (!am) {
		_ERR("failed to get app media");
		return;
	}

	_item_selected_cb(priv, am);
}

static struct listmgr_data *_create_listmgr_data(struct _priv *priv)
{
	struct listmgr_data *data;
	struct play_info_ops *pops;
	struct grid_ops *gops;

	data = calloc(1, sizeof(*data));
	if (!data)
		goto err;

	data->menu_btn = priv->menu_btn;

	data->grid_item_x = GRID_ITEM_X;
	data->grid_item_y = GRID_ITEM_Y;
	data->grid_num_item = GRID_NUM_ITEM;
	data->box_padding = BOX_PADDING;

	pops = calloc(1, sizeof(*pops));
	if (!pops)
		goto err;

	pops->draw = _draw_recent;
	pops->update = _update_recent;
	pops->selected_cb = _recent_selected_cb;
	pops->ops_data = priv;

	gops = calloc(1, sizeof(*gops));
	if (!gops)
		goto err;

	gops->gclass = &_gclass;
	gops->selected_cb = _grid_selected_cb;
	gops->ops_data = priv;

	data->pops = pops;
	data->gops = gops;

	return data;

err:
	_ERR("failed to allocate memory");
	return NULL;
}

static void _update_content_info(struct _priv *priv)
{
	int count;
	char buf[128];

	count = mediadata_get_media_count(priv->md, E_MEDIA_VIDEO);
	if (count < 0) {
		_ERR("failed to get media count");
		return;
	}

	snprintf(buf, sizeof(buf), "%d Movies", count);

	elm_object_part_text_set(priv->layout, PART_CONTENT_INFO, buf);
}

static void _update_content_list(struct _priv *priv)
{
	Eina_List *list;

	if (priv->media_list)
		return;

	list = mediadata_get_list(priv->md, E_LIST_NAME);
	if (!list) {
		elm_object_part_text_set(priv->layout,
				PART_NOCONTENT, TEXT_NOCONTENT);
		return;
	}

	if (!listmgr_update_content_list(priv->listmgr, list)) {
		_ERR("failed to update list area");
		return;
	}

	priv->media_list = list;

	_update_content_info(priv);
}

static void _update_recent_item(struct _priv *priv, int index)
{
	Eina_List *list;
	app_media *am;
	app_media_info *info;

	list = mediadata_get_medialist(priv->md);
	am = eina_list_nth(list, index);
	if (!am) {
		_ERR("failed to get app media");
		return;
	}

	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get app media info");
		return;
	}

	priv->recent_info = am;

	if (!listmgr_update_play_info(priv->listmgr, info))
		_ERR("failed to update recently watched item");
}

static bool _create(layoutmgr *lmgr, void *data)
{
	struct listmgr *listmgr;
	struct listmgr_data *ldata;
	struct mediadata *md;
	struct _priv *priv;
	Evas_Object *base, *layout;

	if (!lmgr) {
		_ERR("failed to get layoutmgr");
		return false;
	}

	if (!data) {
		_ERR("invalid argument");
		return false;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to allocate priv");
		return false;
	}

	priv->menu_btn = (Evas_Object *)data;

	base = layoutmgr_get_base(lmgr);
	if (!base) {
		_ERR("failed to get base object");
		goto err;
	}

	layout = elm_layout_add(base);
	if (!layout) {
		_ERR("failed to create layout");
		goto err;
	}

	if (!elm_layout_file_set(layout, EDJEFILE, GRP_MOVIE_LAYOUT)) {
		_ERR("failed to set layout file");
		goto err2;
	}

	ldata = _create_listmgr_data(priv);
	if (!ldata) {
		_ERR("failed to create listmgr data");
		goto err2;
	}

	listmgr = listmgr_create(layout, (void *)ldata);
	if (!listmgr) {
		_ERR("failed to create listmgr");
		goto err3;
	}

	md = mediadata_create(LIST_MEDIA_COND, E_SOURCE_ALL, E_SORT_NAME);
	if (!md) {
		_ERR("failed to create mediadata");
		listmgr_destroy(listmgr);
		goto err3;
	}

	priv->base = base;
	priv->layout = layout;
	priv->lmgr = lmgr;
	priv->listmgr = listmgr;
	priv->ldata = ldata;
	priv->md = md;

	layoutmgr_set_layout_data(lmgr, LAYOUT_MOVIE, priv);

	if (!listmgr_draw_list_area(priv->listmgr)) {
		_ERR("failed to draw list area");
		mediadata_destroy(md);
		listmgr_destroy(listmgr);
		goto err3;
	}

	return true;

err3:
	free(ldata);
err2:
	evas_object_del(layout);
err:
	free(priv);
	return false;
}

static void _destroy(void *layout_data)
{
	struct _priv *priv;

	if (!layout_data) {
		_ERR("failed to get layout data");
		return;
	}

	priv = layout_data;

	progressbar_destroy(priv->prog);

	mediadata_free_list(priv->media_list);
	mediadata_destroy(priv->md);

	listmgr_destroy(priv->listmgr);
	free(priv->ldata->pops);
	free(priv->ldata->gops);
	free(priv->ldata);

	evas_object_del(priv->layout);

	free(priv);
}

static void _show(void *layout_data)
{
	struct _priv *priv;

	if (!layout_data) {
		_ERR("failed to layout data");
		return;
	}

	priv = layout_data;

	evas_object_show(priv->layout);
	elm_object_part_content_set(priv->base,
			PART_THUMBNAIL_AREA, priv->layout);
}

static void _hide(void *layout_data)
{
	struct _priv *priv;

	if (!layout_data) {
		_ERR("failed to get layout data");
		return;
	}

	priv = layout_data;

	evas_object_hide(priv->layout);
	elm_object_part_content_unset(priv->base, PART_THUMBNAIL_AREA);
}

static void _update(void *layout_data, int update_type, void *data)
{
	struct view_update_data *vdata;
	struct _priv *priv;

	if (!layout_data) {
		_ERR("failed to get layout data");
		return;
	}

	priv = layout_data;
	vdata = data;

	switch (update_type) {
	case UPDATE_CONTENT:
		_update_content_list(priv);
		break;
	case UPDATE_FOCUS:
		if (!vdata) {
			_ERR("invalid argument");
			return;
		}

		listmgr_focus_play_info(priv->listmgr);
		_update_recent_item(priv, vdata->index);
		break;
	case UPDATE_RESUME:
		listmgr_focus_content_list(priv->listmgr,
				priv->cur_index, false);
		break;
	default:
		break;
	}
}

static layout_class _lclass = {
	.layout_id = LAYOUT_MOVIE,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
	.update = _update,
};

layout_class *layout_movie_get_lclass(void)
{
	return &_lclass;
}
