/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <app.h>
#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <inputmgr.h>

#include "define.h"
#include "view.h"

#define MEDIAHUB_WIN_TITLE "Media Hub"

SET_TAG(PACKAGE)

struct _appdata {
	const char *name;
	Evas_Object *win;
};

static Evas_Object *_add_win(const char *name)
{
	Evas_Object *win;
	int w, h;

	win = elm_win_add(NULL, name, ELM_WIN_BASIC);
	if (!win)
		return NULL;

	elm_win_title_set(win, MEDIAHUB_WIN_TITLE);
	elm_win_alpha_set(win, EINA_TRUE);
	elm_win_screen_size_get(win, NULL, NULL, &w, &h);
	evas_object_resize(win, w, h);

	evas_object_show(win);

	return win;
}

static bool _create(void *data)
{
	struct _appdata *ad;
	Evas_Object *win;

	if (!data) {
		_ERR("failed to get data");
		return false;
	}

	ad = data;

	elm_theme_overlay_add(NULL, THEMEFILE);
	elm_config_focus_move_policy_set(ELM_FOCUS_MOVE_POLICY_CLICK);

	win = _add_win(ad->name);
	if (!win) {
		_ERR("failed to create win object");
		return false;
	}

	if (!viewmgr_create(win)) {
		_ERR("failed to initialize viewmgr");
		evas_object_del(win);
		return false;
	}

	viewmgr_add_view(view_base_get_vclass(), NULL);
	viewmgr_add_view(view_viewer_get_vclass(), NULL);
	viewmgr_add_view(view_mplayer_get_vclass(), NULL);

	elm_win_focus_highlight_enabled_set(win, EINA_TRUE);
	elm_win_focus_highlight_style_set(win, STYLE_INVISIBLE);

	ad->win = win;

	return true;
}

static void _terminate(void *data)
{
	struct _appdata *ad;

	if (!data) {
		_ERR("failed to get data");
		return;
	}

	ad = data;

	viewmgr_remove_view(VIEW_BASE);
	viewmgr_remove_view(VIEW_VIEWER);
	viewmgr_remove_view(VIEW_MPLAYER);

	viewmgr_destroy();

	if (ad->win) {
		evas_object_del(ad->win);
		ad->win = NULL;
	}
}

static void _app_control(app_control_h app_control, void *data)
{
	struct _appdata *ad;

	if (!data) {
		_ERR("failed to get data");
		return;
	}

	ad = data;

	if (ad->win)
		elm_win_activate(ad->win);

	viewmgr_push_view(VIEW_BASE);
}

static void _pause(void *data)
{
	inputmgr_enable(EINA_FALSE);
	viewmgr_pause();
}

static void _resume(void *data)
{
	inputmgr_enable(EINA_TRUE);
	viewmgr_resume();
}

int main(int argc, char **argv)
{
	struct _appdata ad;
	ui_app_lifecycle_callback_s cb = {
		.create = _create,
		.terminate = _terminate,
		.app_control = _app_control,
		.pause = _pause,
		.resume = _resume,
	};

	memset(&ad, 0x0, sizeof(ad));
	ad.name = PACKAGE;

	ui_app_main(argc, argv, &cb, &ad);

	return 0;
}
